@@ -57,7 +57,7 @@ class Evaluator(object):
     self._model = model
     self._metrics = {}
     self._evaluators = {}
-    if context.in_graph_mode():
+    if not context.executing_eagerly():
       self.call = function.defun(self.call)
 
   # ---- API for users ----
@@ -90,7 +90,7 @@ class Evaluator(object):
     Only for graph execution.
     @end_compatibility
     """
-    if context.in_eager_mode():
+    if context.executing_eagerly():
       raise RuntimeError("Evaluator.init_variables() not needed when "
                          "eager execution is enabled.")
     return control_flow_ops.group([m.init_variables() for _, m in self.metrics])
@@ -113,7 +113,8 @@ class Evaluator(object):
         with summary_ops.create_file_writer(
             summary_logdir).as_default(), summary_ops.always_record_summaries():
           return self._all_metric_results()
-      if context.in_eager_mode():
+
+      if context.executing_eagerly():
         return f()
       else:
         return function.defun(f)()
@@ -158,16 +159,16 @@ class Evaluator(object):
       @end_compatibility
     """
     summary_logdir = kwargs.pop("summary_logdir", None)
-    if context.in_graph_mode():
-      call_op = self.__call__(dataset.make_one_shot_iterator().get_next(),
-                              *args, **kwargs)
-      init_op = self.init_variables()
-      results_op = self.all_metric_results(summary_logdir)
-      return (init_op, call_op, results_op)
-    # Eager case
-    for example in datasets.Iterator(dataset):
-      self.__call__(example, *args, **kwargs)
-    return self.all_metric_results(summary_logdir)
+    if context.executing_eagerly():
+      for example in datasets.Iterator(dataset):
+        self.__call__(example, *args, **kwargs)
+      return self.all_metric_results(summary_logdir)
+    # Graph construction
+    call_op = self.__call__(dataset.make_one_shot_iterator().get_next(), *args,
+                            **kwargs)
+    init_op = self.init_variables()
+    results_op = self.all_metric_results(summary_logdir)
+    return (init_op, call_op, results_op)
 
   @staticmethod
   def run_evaluation(init_op, call_op, results_op, sess=None):
@@ -192,7 +193,7 @@ class Evaluator(object):
     Only for graph execution.
     @end_compatibility
     """
-    if context.in_eager_mode():
+    if context.executing_eagerly():
       raise RuntimeError("Evaluator.run_evaluation() not supported when "
                          "eager execution is enabled.")
     sess = sess or ops.get_default_session()
