@@ -57,30 +57,6 @@ class LossFunction(object):
     """The inputs to the loss function (excluding the targets)."""
     pass
 
-  @property
-  def input_minibatches(self):
-    """A `list` of inputs to the loss function, separated by minibatch.
-
-    Typically there will be one minibatch per tower in a multi-tower setup.
-    Returns a list consisting of `self.inputs` by default; `LossFunction`s
-    supporting registering multiple minibatches should override this method.
-
-    Returns:
-      A `list` of `Tensor`s representing
-    """
-    return [self.inputs]
-
-  @property
-  def num_registered_minibatches(self):
-    """Number of minibatches registered for this LossFunction.
-
-    Typically equal to the number of towers in a multi-tower setup.
-
-    Returns:
-      An `int` representing the number of registered minibatches.
-    """
-    return len(self.input_minibatches)
-
   def evaluate(self):
     """Evaluate the loss function on the targets."""
     if self.targets is not None:
@@ -474,7 +450,6 @@ class NormalMeanVarianceNegativeLogProbLoss(DistributionNegativeLogProbLoss):
     assert len(variance.shape) == 2, "Expect 2D variance tensor."
     self._mean = mean
     self._variance = variance
-    self._scale = math_ops.sqrt(variance)
     self._targets = targets
     super(NormalMeanVarianceNegativeLogProbLoss, self).__init__(seed=seed)
 
@@ -484,7 +459,7 @@ class NormalMeanVarianceNegativeLogProbLoss(DistributionNegativeLogProbLoss):
 
   @property
   def dist(self):
-    return normal.Normal(loc=self._mean, scale=self._scale)
+    return normal.Normal(loc=self._mean, scale=math_ops.sqrt(self._variance))
 
   @property
   def params(self):
@@ -502,7 +477,7 @@ class NormalMeanVarianceNegativeLogProbLoss(DistributionNegativeLogProbLoss):
 
   @property
   def _fisher_mean_factor(self):
-    return 1. / self._scale
+    return 1. / math_ops.sqrt(self._variance)
 
   @property
   def _fisher_var(self):
@@ -611,36 +586,13 @@ class CategoricalLogitsNegativeLogProbLoss(DistributionNegativeLogProbLoss,
         index in [0, output_size).
       seed: int or None. Default random seed when sampling.
     """
-    self._logits_components = []
-    self._targets_components = []
-    self.register_additional_minibatch(logits, targets=targets)
+    self._logits = logits
+    self._targets = targets
     super(CategoricalLogitsNegativeLogProbLoss, self).__init__(seed=seed)
 
-  def register_additional_minibatch(self, logits, targets=None):
-    """Register an additiona minibatch's worth of parameters.
-
-    Args:
-      logits: Tensor of shape [batch_size, output_size]. Parameters for
-        underlying distribution.
-      targets: None or Tensor of shape [batch_size, output_size].  Each row must
-        be a one-hot vector.
-    """
-    self._logits_components.append(logits)
-    self._targets_components.append(targets)
-
-  @property
-  def _logits(self):
-    return array_ops.concat(self._logits_components, axis=0)
-
-  @property
-  def input_minibatches(self):
-    return self._logits_components
-
   @property
   def targets(self):
-    if all(target is None for target in self._targets_components):
-      return None
-    return array_ops.concat(self._targets_components, axis=0)
+    return self._targets
 
   @property
   def dist(self):
